using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;

namespace GroupingGrid
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Button button1;
		private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl gridGroupingControl1;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private DataSet m_dataSet = null;

		public Form1()
		{
			GridEngineFactory.Factory = new GroupingEngineFactory();
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			CreateTestDataSet();

			SetupGrid();


		}

		private void SetupGrid()
		{
			gridGroupingControl1.BeginUpdate();
			
			gridGroupingControl1.DataSource = m_dataSet.Tables[0];
			
			GridTableDescriptor td = gridGroupingControl1.TableDescriptor;			
		
			gridGroupingControl1.TopLevelGroupOptions.ShowCaption = false;
			gridGroupingControl1.TopLevelGroupOptions.ShowSummaries = false;
			gridGroupingControl1.TableOptions.AllowSelection = GridSelectionFlags.Any;
			gridGroupingControl1.TableDescriptor.AllowEdit = false;
			gridGroupingControl1.TableDescriptor.AllowNew = false;
			gridGroupingControl1.TableDescriptor.AllowRemove = false;

			gridGroupingControl1.TopLevelGroupOptions.IsExpandedInitialValue = true;
			gridGroupingControl1.ChildGroupOptions.IsExpandedInitialValue = true;

			SetupHandlers();
			
			
			td.GroupedColumns.Add("FundingGroup");	
			td.GroupedColumns.Add("SummationBucket");

			td.TopLevelGroupOptions.ShowCaptionSummaryCells = true;
		
			//setup summary row
			GridSummaryRowDescriptor capRow1 = new GridSummaryRowDescriptor();
			capRow1.Name = "Caption";
			capRow1.TitleColumnCount = 0;
			capRow1.Visible = false;
//			for(int i=1; i<m_visibleCols.Length; i++)
//			{
//				if(m_visibleCols[i] == false)
//					continue;
//				string colName = "CapCol" + i.ToString();
//				GridSummaryColumnDescriptor capColumn1 = new GridSummaryColumnDescriptor(colName, SummaryType.DoubleAggregate, td.Columns[i].Name  , "{Sum:g}");
//				capRow1.SummaryColumns.Add(capColumn1);				
//			}
			td.SummaryRows.Add(capRow1);
			gridGroupingControl1.ChildGroupOptions.CaptionText = "{Category}";
			td.ChildGroupOptions.ShowCaptionSummaryCells = true;
			td.ChildGroupOptions.CaptionSummaryRow = "Caption";
			capRow1.Appearance.AnyCell.BackColor = Color.LightGray;
			capRow1.Appearance.AnyCell.Font.Bold = true;

			gridGroupingControl1.Appearance.GroupCaptionCell.BackColor = Color.LightGray;
			gridGroupingControl1.Appearance.GroupCaptionCell.Borders.Top = new GridBorder(GridBorderStyle.Standard);
			gridGroupingControl1.Appearance.GroupCaptionCell.CellType = "Static";
			gridGroupingControl1.Appearance.GroupCaptionCell.Font.Bold = true;

			gridGroupingControl1.EndUpdate(true);	
		}
		private void SetupHandlers()
		{
			//handlers have the life time of grid. should not set for preference change
			gridGroupingControl1.QueryCellStyleInfo += new GridTableCellStyleInfoEventHandler(HandleQueryCellStyleInfo);
		}
		private void HandleQueryCellStyleInfo(object sender, GridTableCellStyleInfoEventArgs e)
		{
			switch(e.TableCellIdentity.TableCellType)
			{
				case GridTableCellType.GroupCaptionCell:
					GridCaptionRow cap = e.TableCellIdentity.DisplayElement as GridCaptionRow;
					if(cap.ParentGroup.GroupLevel == 0)
						return;
					IGridRowHeight gridRow = e.TableCellIdentity.DisplayElement as IGridRowHeight;
				
					if(cap.ParentGroup.GetRecordCount() == 1)
					{
						gridRow.RowHeight = 0;
					}
					break;
			}
		}

		private void button1_Click(object sender, System.EventArgs e)
		{
			double u = 1000000.0; //million
			object[] rec = {
							   "New Bucket", "MBS Group", "New Bucket",
							   -1.3*u,-1.3*u,-1.3*u
						   };
			DataTable dt = m_dataSet.Tables[0];
			gridGroupingControl1.BeginUpdate();
			gridGroupingControl1.Table.TableDirty = true;
			dt.BeginLoadData();
			dt.LoadDataRow(rec, true);		
			dt.EndLoadData();
			dt.AcceptChanges();
			gridGroupingControl1.EndUpdate(true);
		}


		private void CreateTestDataSet()
		{
			m_dataSet = new DataSet();
			DataTable dt = m_dataSet.Tables.Add("Funding"); 
			dt.CaseSensitive = true;
			DataColumn[] keyCols = new DataColumn[3];

			keyCols[0] = dt.Columns.Add("Bucket", typeof(string)); 
			keyCols[1] = dt.Columns.Add("FundingGroup", typeof(string));
			keyCols[2] = dt.Columns.Add("SummationBucket", typeof(string));
			dt.PrimaryKey = keyCols;

			DataColumn col;
			
			int MAX_NUMOF_DATE = 3;	//max date range
			for(int i=0; i<MAX_NUMOF_DATE	; i++)
			{
				col = dt.Columns.Add("date" + i.ToString(), typeof(double));
				col.AllowDBNull = false;
				col.DefaultValue = 0.0;				
			}

			//load data
			double u = 1000000.0; //million
			object[] record1 = {
								   "MBS", "MBS Group", "MBS",
								   10*u,10*u,10*u
							   };
			dt.LoadDataRow(record1, true);
			object[] record2 = {
								   "MBS $", "MBS Group", "MBS $",
								   -7.7*u,-7.7*u,-7.7*u
							   };
			dt.LoadDataRow(record2, true);
			object[] record3 = {
								   "CORP A-AAA", "CORP Group", "CORP",
								   7*u,7*u,7*u
							   };
			dt.LoadDataRow(record3, true);
			object[] record4 = {
								   "CORP BBB", "CORP Group", "CORP",
								   6.1*u,6.1*u,6.1*u
							   };
			dt.LoadDataRow(record4, true);
			object[] record5 = {
								   "CORP B-BB", "CORP Group", "CORP",
								   11.4*u,11.4*u,11.4*u
							   };
			dt.LoadDataRow(record5, true);
			object[] record6 = {
								   "CORP NR", "CORP Group", "CORP",
								   .5*u,.5*u,.5*u
							   };
			dt.LoadDataRow(record6, true);
			object[] record7 = {
								   "CORP $", "CORP Group", "CORP $",
								   -22.6*u,-22.6*u,-22.6*u
							   };
			dt.LoadDataRow(record7, true);
			object[] record8 = {
								   "BILLS", "GOVT", "TSY",
								   14.3*u,14.3*u,14.3*u
							   };
			dt.LoadDataRow(record8, true);
			object[] record9 = {
								   "BONDS", "GOVT", "TSY",
								   7.2*u,7.2*u,7.2*u
							   };
			dt.LoadDataRow(record9, true);
			object[] record10 = {
									"NOTES", "GOVT", "TSY",
									9.1*u,9.1*u,9.1*u
								};
			dt.LoadDataRow(record10, true);
			object[] record11 = {
									"CPI", "GOVT", "TSY",
									1.2*u,1.2*u,1.2*u
								};
			dt.LoadDataRow(record11, true);
			
			object[] record12 = {
									"STRIPS", "GOVT", "TSY",
									2.1*u,2.1*u,2.1*u
								};
			dt.LoadDataRow(record12, true);

			object[] record13 = {
									"TSY $", "GOVT", "TSY $",
									-17*u,-17*u,-17*u
								};
			dt.LoadDataRow(record13, true);

			object[] record14 = {
									"TSY GCF $", "GOVT", "TSY $",
									6*u,6*u,6*u
								};
			dt.LoadDataRow(record14, true);
		
			object[] record15 = {
									"TSY FWD $", "GOVT", "TSY $",
									-1.3*u,-1.3*u,-1.3*u
								};
			dt.LoadDataRow(record15, true);
		}



		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.button1 = new System.Windows.Forms.Button();
			this.gridGroupingControl1 = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
			((System.ComponentModel.ISupportInitialize)(this.gridGroupingControl1)).BeginInit();
			this.SuspendLayout();
			// 
			// button1
			// 
			this.button1.Location = new System.Drawing.Point(120, 24);
			this.button1.Name = "button1";
			this.button1.TabIndex = 0;
			this.button1.Text = "Add a row";
			this.button1.Click += new System.EventHandler(this.button1_Click);
			// 
			// gridGroupingControl1
			// 
			this.gridGroupingControl1.BackColor = System.Drawing.SystemColors.Window;
			this.gridGroupingControl1.Location = new System.Drawing.Point(8, 80);
			this.gridGroupingControl1.Name = "gridGroupingControl1";
			this.gridGroupingControl1.Size = new System.Drawing.Size(728, 472);
			this.gridGroupingControl1.TabIndex = 1;
			this.gridGroupingControl1.Text = "gridGroupingControl1";
			this.gridGroupingControl1.VersionInfo = "4.200.0.37";
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(760, 558);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.gridGroupingControl1,
																		  this.button1});
			this.Name = "Form1";
			this.Text = "Form1";
			((System.ComponentModel.ISupportInitialize)(this.gridGroupingControl1)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

	}
}
